package setpersist.entity;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.TableGenerator;

/**
 *
 * @author name
 */
@Entity
public class Shipment {

    @Id
    @GeneratedValue(strategy = GenerationType.TABLE, generator = "SHIPMENT_GENERATOR")
    @TableGenerator(name = "SHIPMENT_GENERATOR")
    private Integer id;

    @OneToMany(mappedBy="shipment",
            cascade = CascadeType.ALL)
    @OrderBy
    private List<Gee1Leg> gee1Legs;

    @OneToMany(mappedBy="shipment",
            cascade = CascadeType.ALL)
    @OrderBy
    private Set<Agreement> gee1Agreements;

    public void attachBuyer(Person buyer, Long agreementId, EntityManager entityManager){
        // Add agreement
        final Agreement agreement = entityManager.find(Agreement.class, agreementId);
        // Hide the agreement from the waiting list query
        this.gee1Agreements.add(agreement);
        // Find a matching leg
        if(gee1Legs.isEmpty()){
            // Create a new leg.
            createNewLeg(buyer);
        }else{
            // Examine the last leg only.
            final Gee1Leg gee1LegLast = gee1Legs.get(gee1Legs.size() -1);
            // Attach the buyer to the leg
            gee1LegLast.getBuyers().add(buyer);
        }
    }


    private Gee1Leg createNewLeg(Person buyer){
        HashSet <Person> buyers = new HashSet<>();
        buyers.add(buyer);
        final int legNumber;
        if(gee1Legs.isEmpty()){
            legNumber = 1;
        }else{
            legNumber = gee1Legs.get(gee1Legs.size() - 1).getLegNumber() + 1;
        }
        Gee1Leg gee1Leg = new Gee1Leg(this, legNumber, buyers);
        this.gee1Legs.add(gee1Leg);
        return gee1Leg;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public Integer getId() {
        return id;
    }

    public List<Gee1Leg> getGee1Legs() {
        return gee1Legs;
    }

    public void setGee1Legs(List<Gee1Leg> gee1Legs) {
        this.gee1Legs = gee1Legs;
    }

    public Set<Agreement> getGee1Agreements() {
        return gee1Agreements;
    }

    public void setGee1Agreements(Set<Agreement> gee1Agreements) {
        this.gee1Agreements = gee1Agreements;
    }

}
