package setpersist.session;

import setpersist.entity.Shipment;
import setpersist.entity.Gee1Leg;
import setpersist.entity.Agreement;
import setpersist.entity.Person;
import java.util.List;
import java.util.Set;
import org.junit.After;
import org.junit.AfterClass;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author name
 */
public class Gee1SessionMobileTest extends PersistentIntegrationTest {

    public Gee1SessionMobileTest() {
    }

    private static Gee1SessionMobile instance;

    @BeforeClass
    public static void setUpClass() throws Exception {
        instance = new Gee1SessionMobile(em);
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() throws Exception {
    }

    @After
    public void tearDown() throws Exception {
    }

    /**
     * Default generated attaches a single passenger and tests
     */
    @Test
    public void testChangeLeg() throws Exception {
        System.out.println("changeLeg");

        // pre-test
        Person customerMember = new Person();
        em.persist(customerMember);
        Person providerMember = new Person();
        em.persist(providerMember);
        // Passenger requests agreement
        Agreement agreement = new Agreement();
        em.persist(agreement);
        agreement.setBuyer(customerMember);
        agreement.setSeller(providerMember);
        em.flush();
        em.clear();
        Shipment shipmentFound = instance.createShipment();
        em.flush();
        Integer tripId = shipmentFound.getId();
        em.clear();
        assertNotNull(tripId);
        shipmentFound = em.find(Shipment.class, tripId);
        assertNotNull(shipmentFound);

        // Test

        instance.changeLeg(tripId, agreement.getSeller(), agreement.getBuyer().getId(), agreement.getId());
        agreement = em.find(Agreement.class, agreement.getId());
        // Do again to ensure that agreement is not added again.
        instance.changeLeg(tripId, agreement.getSeller(), agreement.getBuyer().getId(), agreement.getId());
        em.flush();
        em.clear();
        // Check the database
        shipmentFound = em.find(Shipment.class, tripId);

        List<Gee1Leg> gee1Legs = shipmentFound.getGee1Legs();
        assertFalse(gee1Legs.isEmpty());
        assertEquals(1, gee1Legs.size());
        Gee1Leg gee1Leg = gee1Legs.get(0);
        Set<Person> passengersOnLeg = gee1Leg.getBuyers();
        assertFalse(passengersOnLeg.isEmpty());
        Person passengerFound = passengersOnLeg.iterator().next();
        assertNotNull(passengerFound);
        assertEquals(agreement.getBuyer(), passengerFound);
        Set<Agreement> gee1AgreementsOnTrip = shipmentFound.getGee1Agreements();
        assertFalse(gee1AgreementsOnTrip.isEmpty());// bt-fixme: Agreements not added?
        assertEquals(1, gee1AgreementsOnTrip.size());
        Agreement gee1AgreementOnTrip = gee1AgreementsOnTrip.iterator().next();
        assertEquals(agreement, gee1AgreementOnTrip);
    }
}