package setpersist.session;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;

/**
 * Establishes persistence context. FOr each test, creates a transaction and
 * rolls it back
 *
 * @author Administrator
 */
public abstract class PersistentIntegrationTest {

    public static String PERSISTENCE_UNIT_NAME = "SetPersistDropCreatePu";
    private static EntityManagerFactory emf;
    protected static EntityManager em;
    private static EntityTransaction trans;

    @BeforeClass
    public static void setUpClassBefore() throws Exception {
        emf = Persistence.createEntityManagerFactory(PERSISTENCE_UNIT_NAME);
        em = emf.createEntityManager();
    }

    @AfterClass
    public static void tearDownClassAfter() throws Exception {
        if (em != null) {
            em.close();
        }
        if (emf != null) {
            emf.close();
        }
    }

    @Before
    public void setUpBefore() throws Exception {
        try {
            trans = em.getTransaction();
            trans.begin();
        } catch (Exception ex) {
            throw ex;// for debugging.
        }
    }

    @After
    public void tearDownAfter() {
        if (em.getTransaction().isActive()) {
            // Flush to catch any database issues eg duplicate inserts
            try {
                em.flush();
            } finally {
                trans.rollback();
            }
        }
    }
}